# -*- coding: utf-8 -*-
"""
"""
import sys
import os

import configargparse
import pkg_resources

ROBOT_LISTENER_API_VERSION = 2

def robotL10NListener():
    return

def end_test(name, attrs):
    if attrs['status'] == 'FAIL':
        print(f'Test "{name}" failed: {attrs["message"]}')
        input('Press enter to continue.')

def init_config():
    p = configargparse.ArgParser(default_config_files=[".robotLocalization.yml"])
    # disagnose options
    p.add_argument("-v", "--verbose", help="enable verbose mode", env_var="VERBOSE",
        default=False, action='store_true')
    p.add_argument("-d", "--debug", env_var="DEBUG", default=False, action='store_true')
    p.add_argument("-n", "--dry_run", help="enable dry run mode.", action='store_true')
    p.add_argument("--locale", help="specify locale", default="en", action='store')

    # cli operand

    c = vars(p.parse_args())

    if c["verbose"]:
        print(f"config={c}")

    return c, p

def main():
    # config, argparser = init_config()
    grc = 0

    sys.stderr.write("successfully completed.\n" if grc == 0 else f"failed. rc={grc}\n")

    sys.exit(grc)



import os
import sys
import re
import json
import traceback

from jproperties import Properties

from robotLocalization import util
from robotLocalization import loader_xml

var_pat = re.compile(r'\$\{(?P<var>.*?)\}')
def load_properties(path,encoding='utf-8',locale="en",verbose=False):
    properties = {}
    prop = Properties()
    placeholder_pat = re.compile(r'.*\{.*?\}.*')  # detect placeholders

    basename = os.path.basename(path)
    filename , fext = os.path.splitext(basename)
    if locale != "en":
        path = util.locale_lookup(path,locale,verbose=verbose)
        if verbose:
            print (f"locale fallback path={path}")

    if not os.path.exists(path):
        return properties
    try:
        basename1 = os.path.basename(path)
        if verbose:
            print(f"Loading {basename1} from {path}...")
        with open(path,'rb') as pr:
            prop.load(pr,encoding)
        base_properties = {k: v[0] for k, v in prop.items()}

        for key in base_properties:
            value = base_properties[key]
            # check quote character here.
            # U+0060  --> U+2019
            m = placeholder_pat.search(value)
            if m:
                val0 = tokenize(value)
                if val0 and len(val0)>=1:
                    #print (f"value = {value} --> {val0[0]}")
                    value = val0[0]
                else:
                    # discard partial strings
                    continue

            if key in ["***" , ""] :
                continue
            m = var_pat.match(key)
            if m:
                var = m.group('var')
                properties[var] = value
            else:
                properties[key] = value

        return properties
    except:
        traceback.print_exc()
        sys.stderr.write(f"Unable to open {path}\n")

    return properties

def load_json(path,encoding='utf-8',locale="en",verbose=False):
    # basename = os.path.basename(path)
    # filename , fext = os.path.splitext(basename)
    if locale != "en":
        path = util.locale_lookup(path,locale,verbose=verbose)
        if verbose:
            print (f"locale fallback path={path}")
    with open(path,encoding='utf-8') as fp:
        properties = json.load(fp)
    return properties

def load_m(path,encoding='utf-8',locale="en",verbose=False):
    properties = {}
    if locale != "en":
        path = util.locale_lookup(path,locale,verbose=verbose)
        if verbose:
            print (f"locale fallback path={path}")
    with open(path,encoding='utf-8') as fp:
        lines = fp.readlines()
    return properties

def load_tkmsg(path,encoding='utf-8',locale="en",verbose=False):
    properties = {}
    if locale != "en":
        path = util.locale_lookup(path,locale,verbose=verbose)
        if verbose:
            print (f"locale fallback path={path}")
    with open(path,encoding='utf-8') as fp:
        lines = fp.readlines()
    return properties

def load_smd(path,encoding='utf-8',locale="en",verbose=False):
    return load_properties(path,encoding=encoding,locale=locale,verbose=verbose)

def load_strings(file):
    properties = {}
    return properties

loaders = { "properties":load_properties, "json":load_json, "m":load_m,
    "tkmsg":load_tkmsg, "smd":load_properties, "strings":load_strings,
    "xml":loader_xml.load_xml }

def load_files(file,encoding='utf-8',locale="en",verbose=False):
    properties = {}

    if not file:
        return None
    fext = os.path.splitext(file)
    if not fext[1][0] == ".": # file has an extention
        sys.stderr.write(f"Unknown file type: {fext}\n")
        return None
    if not os.path.exists(file):
        sys.stderr.write(f"Error: {file} does not exist.\n")
        return None
    filetype = fext[1][1:]

    if filetype in loaders:
        if verbose:
            print (f"Loading {file} using \"{filetype}\" loader for {locale}...")
        properties = loaders[filetype](file,locale=locale,verbose=verbose)

    return properties

def tokenize(value):
    # remove placeholder and tokenize string
    # input: The table was successfully imported on {0} and is ready for use.
    # output: "The table was successfully imported on", "and is ready for use."
    value0 = re.sub(r'\s*"?{.*?}"?\s*','  ',value)
    vallist = value0.split('  ')
    if len(vallist) > 1:
        vallist2 = [v for v in vallist if v]
        return vallist2
    return None

if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description='Loader test')
    parser.add_argument("--debug",action="store_true",dest="debug",
                    help="enable debug output")
    parser.add_argument("--locale",action="store",dest="locale",
                    default="en",help="specify use locale")
    parser.add_argument("files",action="append",
                    help="specify files")
    args = parser.parse_args()

    files = args.files
    files = []
    files.append("../DMM/row-sets-service/src/main/resources/com/sas/rowsets/support/L10nMessages.properties")
    #files.append("../DMM/reference-data-manager/src/main/doc/en/dmrdmmid.properties")
    #files.append("../HTMLCommons/i18n/localization-repository/bundles/nova/nova-svg/SASDesign-Icon-gui-icu.properties")
    files.append("../RiskCirrus/convoy/risk-cirrus-pcpricing/ui/i18n/en/Cycle.json")
    files.append("../ViyaMVA/cvs_sas/tsmsg/msg/en/tsmodmsg.m")
    files.append("../ViyaMVA/cvs_sas/tkhpstat/msg/en/tkmpc.tkmsg")

    args.locale = "zh_TW"

    for file in files:
        properties = load_files(file,locale=args.locale,verbose=True)
        if not properties:
            sys.stderr.write("Properties loading failed.\n")
            continue
        max_count = 10
        for key,value in properties.items():
            if max_count >0:
                max_count -= 1
            else:
                break
            print (f"{max_count} {key} = {value}")

    sys.exit(0)



import os
import sys

import xml.etree.ElementTree as ET

from robotLocalization import util

def load_xml(path,encoding='utf-8',locale="en",verbose=False):
    if encoding != 'utf-8':
        print ("Warning: xml loader only supports utf-8 encoding.")

    properties = {}
    nprop = 0
    if locale != "en":
        path = util.locale_lookup(path,locale,verbose=verbose)
        if verbose:
            print (f"locale fallback path={path}")

    if not os.path.exists(path):
        return properties

    try:
        tree = ET.parse(path)
    except ET.ParseError as e:
        sys.stderr.write(f"Fatal: XML error while parsing {path}\n{e}\n")
        return {}

    root = tree.getroot()
    if root.tag == 'properties':
        for child in root:
            if child.tag == 'entry':
                value = child.text

                if len(child) > 0: # fix TNGBUGS-1964
                    if not value:
                        value = ""
                    for gc in child:
                        value += ET.tostring(gc,encoding='unicode')
                # decode NCR
                value = value.replace('&gt;','>')
                value = value.replace('&lt;','<')
                properties[child.attrib['key']] = value
                nprop += 1
            else:
                sys.stderr.write(f"invalid tag:{child.tag}\n")
    else:
        sys.stderr.write(f"Warning Unrecotnized xml file {path}. Ignored.\n")

    return properties

if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description='Loader test')
    parser.add_argument("--debug",action="store_true",dest="debug",
                    help="enable debug output")
    parser.add_argument("--locale",action="store",dest="locale",
                    default="en",help="specify use locale")
    parser.add_argument("files",action="append",
                    help="specify files")
    args = parser.parse_args()

    files = args.files
    files = []
    files.append("../CIGTL/CustomerIntelligence/Analytic/Lambdas/mkt-analytic-lambda-athena/src/insights/locales/insights.xml")

    args.locale = "ja"

    for file in files:
        properties = load_xml(file,locale=args.locale,verbose=True)
        if not properties:
            sys.stderr.write("Properties loading failed.\n")
            continue
        max_count = 10
        for key,value in properties.items():
            if max_count >0:
                max_count -= 1
            else:
                break
            print (f"{max_count} {key} = {value}")

    sys.exit(0)



#!/usr/bin/env python3
import os
import re

from . import api
from . import loader
from . import util

prop_pat = re.compile(r'(?P<key>\w+)\s*=\s*(?P<value>.*?)$')
def prioritize_files(afile,*args,verbose=False):
    proplist = {}
    properties = {}
    dirlist = {}
    stringlist = {}

    print (f"Analyzing {afile}...")
    robot, values, sysprop = api.extract(afile)
    vallist = values.split("\n")
    for val in vallist:
        if not val:
            continue
        m = prop_pat.match(val)
        if not m:
            print (f"invalid value {val}...")
            continue
        properties[m.group('key')] = m.group('value')

    print (f"The following strings are found in {afile}:")
    for key in properties:
        print (f"\t{key} = {properties[key]}")
        stringlist[key] = { "value":properties[key], "in":[] }
    print ("")

    pathlist, path_options = util.get_pathlist(*args)
    if path_options["verbose"]:
        verbose = path_options["verbose"]

    for path in pathlist:
        print (f"Checking {path}...")
        basefiles = {}
        variables = {}

        basefiles0 = util.get_basefiles(path,verbose=verbose)
        basefiles.update(basefiles0)

        npropfiles = 0
        for base in basefiles:
            npropfiles += 1
            p = loader.load_properties(basefiles[base]['en'],verbose=False)
            nprop = len(p)
            if not nprop:
                print (f"Empty file {basefiles[base]['en']}".replace(os.path.sep,'/'))
                continue
            dirname = os.path.dirname(basefiles[base]['en'])
            if dirname not in dirlist:
                dirlist[dirname] = { "count":0 , "match":0 }
            dirlist[dirname]["count"] += 1
            dirlist[dirname]["match"] += get_hitcount(base,properties,p)
            count = get_hitlist(base,stringlist,p)

        print (f"{npropfiles} basefiles found in {path}.")

    print ("")
    ranking = []
    for dirname in dirlist:
        if dirlist[dirname]["match"] > 0:
            ranking.append (f'{dirlist[dirname]["match"]:3}: {dirname}'.replace(os.path.sep,'/'))
    ranking.sort(reverse=True)
    for line in ranking:
        print(line)

    scount = 0
    if len(stringlist):
        for key in stringlist:
            if len(stringlist[key]["in"]) == 0:
                if scount == 0:
                    print ("\n=== Strings required new localization ===")
                scount += 1
                print (f'\t{key} = {stringlist[key]["value"]}')
    if not scount:
        print ("Note: All properties can be loaded from the specified repositories.")

    nprop = len(properties)
    print (f"\nTotal strings found in {afile}: {nprop}")
    print (f"New strings found: {scount} out of {nprop}")

    rcount = 0
    prodpath = {}
    for key in stringlist:
        if len(stringlist[key]["in"]) == 1:
            #print (f'{key} = {stringlist[key]["value"]} {stringlist[key]["in"][0]}')
            rcount += 1
            if stringlist[key]["in"][0] not in prodpath:
                prodpath[stringlist[key]["in"][0]] = 0
            prodpath[stringlist[key]["in"][0]] += 1

    if len(prodpath):
        print ("\n=== Unique PATH ===")
        for path0 in prodpath:
            print(f"{path0} {prodpath[path0]}".replace(os.path.sep,'/'))

    # p = api.load_properties(path=path,verbose=verbose)
    # if p:
    #     proplist[path] = { "dict":p , "count":0 }

    return proplist

# nfound = analyze.search(config["search_key"],*config["path"],verbose=config["verbose"])
def search(string,*args,locale="en",verbose=False,search_key=False,search_trans=False,
            search_english=False,search_startswith=False,search_contains=False):
    nfound = 0
    mode = "String" if not search_key else "Key"
    tlocale = None

    pathlist, path_options = util.get_pathlist(*args)
    if path_options["verbose"]:
        verbose = path_options["verbose"]

    print (f"Searching for \"{string}\" ...")
    nfound = 0
    nprop = 0
    nfiles = 0

    if search_trans:
        if locale == "en": # specification error
            print ("Warning: --search_trans requires non-English locale.  Specify non-English locale with --locale option.")
            return 0
        tlocale = locale
        locale = "en"
        trans = {}
    elif search_english:
        if locale == "en":
            print ("Warning: --search_english requires non-English locale.  Specify non-English locale with --locale option.")
            return 0
        tlocale = "en"
        trans = {}

    for path in pathlist:
        basefiles = {}
        variables = {}

        if path_options["filetype"]:
            basefiles0 = util.get_basefiles(path,filetype=path_options["filetype"],verbose=verbose)
        else:
            basefiles0 = util.get_basefiles(path,verbose=verbose)
        basefiles.update(basefiles0)

        for base in basefiles:
            nfiles += 1
            p = loader.load_files(basefiles[base]["en"],locale=locale,verbose=False)
            if search_trans or search_english:
                q = loader.load_files(basefiles[base]["en"],locale=tlocale,verbose=False)
                if not q or not len(q): # translation is not available
                    continue
            nprop += len(p)
            nhit0 = 0
            for k in p:
                value = p[k] if not search_key else k
                cond = (string == value) if not search_startswith else value.startswith(string)
                if not cond and search_contains:
                    cond = value.find(string)>=0
                if cond:
                    if not nhit0:
                        print (f"{basefiles[base]['en']}")
                    nhit0 += 1
                    print (f"\t{k} = {p[k]}")
                    if search_trans or search_english:
                        if k in q:
                            slen = len(k)
                            print (f"\t{slen*' '} = {q[k]}")
                            if q[k] not in trans:
                                trans[q[k]] = 0
                            trans[q[k]] += 1
                        else:
                            print ("Translation not found.")

            nfound += nhit0
    if nfound and (search_trans or search_english):
        if len(trans) >1: # multiple translation found
            print ("Inconsistent translations found:")
            for tr in trans:
                print (f"\t{tr} : {trans[tr]}")
    print (f"{mode} \"{string}\" found {nfound} times in {nfiles} files.")

    return nfound

def search_key(key,*args,locale="en",verbose=False):
    return search(key,*args,locale=locale,verbose=verbose,search_key=True)

def search_startswith(string,*args,locale="en",verbose=False):
    return search(string,*args,locale=locale,verbose=verbose,search_startswith=True)

def search_trans(string,*args,locale="en",verbose=False):
    return search(string,*args,locale=locale,verbose=verbose,search_trans=True)

def search_english(string,*args,locale="en",verbose=False):
    return search(string,*args,locale=locale,verbose=verbose,search_english=True)

def search_contains(string,*args,locale="en",verbose=False):
    return search(string,*args,locale=locale,verbose=verbose,search_contains=True)

def get_hitcount(path,properties,p):
    count = 0
    for prop in properties:
        value = properties[prop]
        for p0 in p:
            if value == p[p0]:
                count += 1
                #print (f"\"{value}\" found in {p0}/{path}")
    return count

def get_hitlist(path,stringlist,p):
    count = 0
    for key in stringlist:
        value = stringlist[key]["value"]
        for p0 in p:
            if value == p[p0]:
                count += 1
                stringlist[key]["in"].append(path)
                # print (f"\"{value}\" found in {p0}@{path}")
    return count



# -*- coding: utf-8 -*-
"""
"""
import sys
import os

import configargparse
import pkg_resources

from robotLocalization import api
from robotLocalization import loader
from robotLocalization import analyze
from robotLocalization import util

def get_variables(locale,*args,verbose=False):
    variables = {}
    arglist = []

    arglist, path_options = util.get_pathlist(*args)
    if path_options["verbose"]:
        verbose = path_options["verbose"]
    # if verbose and path_options["filetype"]:
    #     sys.stderr.write("Warning: filetype option is not supported yet.")

    for path in arglist:
        if os.path.isdir(path):
            p = api.traverse_tree(path,locale,
                    filetype=path_options["filetype"] if path_options["filetype"] else "properties",
                    verbose=verbose)
            # p = api.traverse_tree(path,locale,
            #         filetype=path_options["filetype"],
            #         verbose=verbose)
            variables.update(p)  # old entries will be replaced by later one.
        else:
            p = loader.load_files(path,verbose=verbose)
            if locale != "en":
                l = loader.load_files(path,locale=locale,verbose=verbose)
                p.update(l)
            variables.update(p)
    return variables

def init_config():
    p = configargparse.ArgParser(default_config_files=[".robotLocalization.yml"])
    # disagnose options
    p.add_argument("-v", "--verbose", help="enable verbose mode", env_var="VERBOSE", default=False, action='store_true')
    p.add_argument("--terse", help="enable terse mode", env_var="TERSE", default=False, action='store_true')
    p.add_argument("-d", "--debug", env_var="DEBUG", default=False, action='store_true')
    p.add_argument("-n", "--dry_run", help="enable dry run mode.", action='store_true')
    p.add_argument("--test", help="enable test flag", action='store_true')

    # cli operand
    p.add_argument("--dump", help="list all of the properties", action='store_true')
    p.add_argument("--search", help="specify a atring to search", action='store')
    p.add_argument("--search_key", help="specify a atring key to search", action='store')
    p.add_argument("--search_startswith", help="specify a atring to search translation", action='store')
    p.add_argument("--search_trans", help="specify a atring to search translation", action='store')
    p.add_argument("--search_english", help="specify a atring to search English string", action='store')
    p.add_argument("--search_contains", help="specify a atring containing the specified string", action='store')
    p.add_argument("--extract", help="specify a keyword file to externalize", action='store')
    p.add_argument("--outp","--output_properties", help="specify a output properties file", action='store')
    p.add_argument("--outr","--output_robot", help="specify a output robot test file", action='store')
    p.add_argument("--outb","--output_bundle", help="specify a product properties bundle file", action='store')
    p.add_argument("--use_keys", "--use_bundle", help="use the existing string keys from specified bundles", action='store_true')
    p.add_argument("--analyze", help="specify a variable file to analyze", action='store')

    # bundle operation - need more idea. how to generate language specific bundle??

    # process options
    p.add_argument("--locale", help="specify locale", default="en", action='store')
    p.add_argument("path", nargs='*', help="specify pathes or files", action='store')
    p.add_argument("--ppath", "--project_path",help="specify project path list or files", action='store',env_var="PROJECT_PATH")
    p.add_argument("--bundle_locales", help="specify locales for product bundles", action='store')
    p.add_argument("--add", help="specify a string key", action='append')
    #p.add_argument("--filetype", help="specify a filetype list", action='store')

    c = vars(p.parse_args())

    if c["verbose"]:
        print(f"config={c}")

    if not c["path"]:
        if c["ppath"]:
            c["path"] = c["ppath"].split(":")

    if c["use_keys"] and not c["path"]:
        sys.stderr.write("Warning: --use_keys option is used without project path list. --use_keys option ignored.\n")

    return c, p

def main():
    config, argparser = init_config()
    grc = 0

    if config["analyze"]:
        if not os.path.exists(config["analyze"]):
            sys.stderr.write("Error: Specified files does not exist.\n")
            sys.exit(9)
        variables = analyze.prioritize_files(config["analyze"],*config["path"],verbose=config["verbose"])
        sys.exit(0)
    elif config["search"]:
        nfound = analyze.search(config["search"],*config["path"],locale=config["locale"],verbose=config["verbose"])
        sys.exit(nfound)
    elif config["search_key"]:
        nfound = analyze.search_key(config["search_key"],*config["path"],locale=config["locale"],verbose=config["verbose"])
        sys.exit(nfound)
    elif config["search_startswith"]:
        nfound = analyze.search_startswith(config["search_startswith"],*config["path"],locale=config["locale"],verbose=config["verbose"])
        sys.exit(nfound)
    elif config["search_trans"]:
        nfound = analyze.search_trans(config["search_trans"],*config["path"],locale=config["locale"],verbose=config["verbose"])
        sys.exit(nfound)
    elif config["search_english"]:
        nfound = analyze.search_english(config["search_english"],*config["path"],locale=config["locale"],verbose=config["verbose"])
        sys.exit(nfound)
    elif config["search_contains"]:
        nfound = analyze.search_contains(config["search_contains"],*config["path"],locale=config["locale"],verbose=config["verbose"])
        sys.exit(nfound)

    variables = get_variables(config["locale"],*config["path"],verbose=config["verbose"])

    if config["dump"]:
        for var in variables:
            print (f"{var} = {variables[var]}")
    nprop = len(variables)
    if config["path"]:
        print (f"{nprop} variables loaded totally.")

    if config["extract"]:
        print (f'Extracting strings for \"{config["extract"]}\" ...')
        if config["use_keys"]:
            robotfile, propfile, sysprop = api.extract(config["extract"],variables,
                verbose=config["verbose"])
        else:
            robotfile, propfile, sysprop = api.extract(config["extract"],
                verbose=config["verbose"])
        if not robotfile:
            sys.exit(8)

        if config["add"]: # need extra strings?
            for add0 in config["add"]:
                addkey = api.bundle_reference(add0,variables)
                if len(addkey) >0:
                    sysprop.extend(addkey)
                else:
                    varname0 = api.varname(add0)
                    propfile += f"{varname0} = {add0}\n"
                    print (f"String \"{add0}\" not available in the specified path list, generated in propertiles file.")

        if config["outp"]:
            if propfile == "":
                print ('No localizable strings found. Empty file generated.')
            print (f'Output properties file: {config["outp"]}...')
            if not config["dry_run"]:
                rc = api.write_file(config["outp"],propfile,"Properties file")
                if not rc:
                    print (f"Unexpect error in write_files: rc={rc}")
            else:
                print ("--dry_run mode specified. ")
                print ("{propfile}")
        else: # dump mode
            if len(propfile):
                print (f"The following properties are extracted:\n{propfile}")
            else:
                print ("No properties need to be extracted.")

        if robotfile == "":
            print ('Empty robot file.')
        else:
            if config["outr"]:
                print (f'Output robot file: {config["outr"]}...')
                if not config["dry_run"]:
                    rc = api.write_file(config["outr"],robotfile,"Robot file")
                    if not rc:
                        print (f"Unexpect error in write_files: rc={rc}")
                else:
                    print ("--dry_run mode specified. ")
                    print (f"{robotfile}")
            else: # dump mode
                if config["verbose"]:
                    print ("Specify ---outr options to store the internationalized robot file.")

        if config["outb"]:
            if len(sysprop) == 0:
                print ('No product properties used. Product bundle file is not generated.')
            else:
                print (f'Output bundle properties file: {config["outb"]}...')
            if not config["dry_run"]:
                syspropfile = api.generate_bundle(sysprop,variables=variables)
                rc = api.write_file(config["outb"],syspropfile,"Propertiles Bundle")
                if not rc:
                    print (f"Unexpect error in write_files: rc={rc}")
                if config["bundle_locales"]:
                    fext = os.path.splitext(config["outb"])
                    #print (f"{fext}")
                    syspropfile = api.generate_bundle(sysprop,variables=variables)
                    bundle_locales = config["bundle_locales"].split(",")
                    for loc in bundle_locales:
                        print(f"Bundling properties for {loc}...")
                        loc_variables = get_variables(loc,*config["path"],verbose=config["verbose"])
                        syspropfile = api.generate_bundle(sysprop,variables=loc_variables)
                        rc = api.write_file(fext[0] + f"_{loc}" + fext[1],syspropfile,f"Propeprties Bundle for {loc}")
                        #print (f"{syspropfile}")
            else:
                print ("--dry_run mode specified. ")
                print (f"{syspropfile}")
        else: # dump mode
            syspropfile = api.generate_bundle(sysprop,variables=variables)
            if len(syspropfile):
                print (f"The following properties are extracted from project files:\n{syspropfile}")
            # else:
            #     print ("No properties need to be extracted from project files.")

    if config["verbose"]:
        sys.stderr.write("successfully completed.\n" if grc == 0 else f"failed. rc={grc}\n")

    sys.exit(grc)



#!/usr/bin/env python3
import sys
import os
import re
import codecs

from robotLocalization import util
from robotLocalization import loader

def traverse_tree(rootdir,locale="en",filetype="properties",verbose=False):
    basefiles = util.get_basefiles(rootdir,filetype=filetype,verbose=verbose)

    properties = {}

    for file in basefiles:
        p = loader.load_files(basefiles[file]["en"],verbose=verbose)
        if locale != "en":
            lfile = util.locale_lookup(basefiles[file]["en"],locale)
            if lfile and lfile != basefiles[file]["en"]:
                l = loader.load_files(lfile)
                p.update(l)
            else:
                if lfile:
                    relpath = os.path.relpath(lfile)
                    if verbose:
                        print (f'Warning: properties file "{relpath}" is not localized for the target locale "{locale}".')
                else:
                    if verbose:
                        print (f'Warning: file "{file}" is not recognized for the locale "{locale}".')

        properties.update(p)

    return properties

text_pat = re.compile(r'''(text\(\)\s*|@aria-label\s*|@placeholder\s*|@title\s*|\.\s*?)([=,]\s*)(['"])(?P<T>.*?)\3''')
value_pat = re.compile(r'(?<=\s\s)(?P<value>\S.*)(?=\s\s)')
value2_pat = re.compile(r'(?<=\s\s)(?P<value>\S.*)')
I18NOK_pat = re.compile(r"#\s*i18n:OK")
placeholder_pat = re.compile(r'\(_+\w+_+\)')
def extract(path,variables=None,verbose=False):
    propfile = ""
    robotfile = ""
    syspropfile = ""
    sysprop = []

    if not os.path.exists(path):
        print (f"{path} does not exist.")
        return None, None, None

    with codecs.open(path,'r',encoding='utf-8') as fp:
        lines = fp.readlines()

    keys = {}
    for line in lines:
        newline = line
        m = I18NOK_pat.search(line)
        if m:
            robotfile += line
            continue
        m = value_pat.search(line)
        if not m:
            m = value2_pat.search(line)
        if m:
            value = m.group('value')
            if not value.startswith('xpath'):
            # process variables
                var = varname(value)
                if variables:
                    sysprop.extend(bundle_reference(value,variables)) # call before var_reference
                value , nhit = var_reference(value,variables,hint="label",verbose=verbose)
                if nhit:
                    print (f"var={var} value={value} {m.start()} {m.end()}")
                    line = line[:m.start()] + "${" + value + "}" + line[m.end():]
                else:
                    line = line[:m.start()] + "${" + var + "}" + line[m.end():]
                    if var not in keys:
                        propfile += f"{var} = {value}\n"
                        keys[var] = 0
                    keys[var] += 1
                robotfile += line
                continue

        seg = text_pat.finditer(newline)
        if seg:
            count = 0
            lastchar = 0
            edlist = []
            for m in iter(seg):
                strcls = m.group(1)
                #print (f"strcls={strcls}")
                value = m.group(4)
                var = varname(value)
                n = placeholder_pat.match(value)
                if n:
                    #print(f"placeholder={value}")
                    continue
                if value[0:2] == '${' or value == '(_____placeholder_____)': # var reference
                    continue
                #newline = newline[:m.start()] + m.group(1) + m.group(2) + "'" + var + "'" + newline[m.end():]
                if variables:
                    sysprop.extend(bundle_reference(value,variables)) # call before var_reference

                value , nhit = var_reference(value,variables,hint=strcls,verbose=verbose)
                if nhit:
                    # print (f"Indirect reference: {var} = {value}")
                    var = value
                    keys[var] = 0

                edlist.append({"begin":m.start(), "end":m.end(),"var":var , "g1":m.group(1), "g2":m.group(2)})
                if var not in keys:
                    propfile += f"{var} = {value}\n"
                    keys[var] = 0
                keys[var] += 1
                count += 1
            #print (f"keys={keys}")
            if count == 0:
                robotfile += line
            else:
                edlist.reverse()
                for ix in edlist:
                    newline = newline[:ix["begin"]] + ix["g1"] + ix["g2"] + '"${' + ix["var"] + '}"' + newline[ix["end"]:]
                #print (f"edlist={edlist}")
                robotfile += newline
            #print (f"   line={line}\nnewline={newline}")
        else:
            print (f"Return from finditer is None: {line}")
            robotfile += line

    return robotfile, propfile, sysprop

def varname(label):
    label = label.lower().replace(' ','_')
    label = label.replace(':','C')
    label = label.replace('?','Q')
    label = label.replace('.','P')
    label = label.replace(',','M')
    label = label.replace('"','D')
    label = label.replace('{','L')
    label = label.replace('}','R')
    label = label.replace('(','S')
    label = label.replace(')','T')
    label = label.replace('$','V')
    label = label.replace("'",'A')
    #label = "${" + label +  "}"
    return label

def var_reference(label,variables=None,hint=None,verbose=False):
    # hint: text(), ., @placeholder, @aria-label, @title
    chrclass = { "placeholder":[], "label":[], "title":[], "tooltip":[], "icon":[] }
    nhit = 0
    candidates = []

    if not variables:
        return label, 0

    for key in variables:
        if variables[key] == label:
            nhit += 1
            candidates.append(key)

    if len(candidates)==0:
        return label, 0
    if len(candidates)==1:  # only one found
        label = candidates[0]
        return label, 1
    # clasify keys
    for key in candidates:
        lkey = key.lower()
        if lkey.find("icon.label")>=0:
            chrclass["icon"].append(key)
        elif lkey.find("title")>=0:
            chrclass["title"].append(key)
        elif lkey.find("placeholder")>=0:
            chrclass["placeholder"].append(key)
        elif lkey.find("label")>=0:
            chrclass["label"].append(key)
        elif lkey.find("tooltip")>=0:
            chrclass["tooltip"].append(key)
        else:
            chrclass["label"].append(key)

    def select(key,list):
        ncount = len(chrclass[key])
        if ncount >=1:
            if ncount >1 and verbose:
                print (f'Warning: {ncount} keys found for {key}:')
                for st in chrclass[key]:
                    print (f"\t{st}")
            return chrclass[key][0], ncount
        return None, ncount

    if hint == "@title":
        label0, count0 = select("title",candidates)
        if count0 > 0:
            return label0, count0
    elif hint == "@placeholder":
        label0, count0 = select("placeholder",candidates)
        if count0 > 0:
            return label0, count0
    elif hint == "@aria-label":
        label0, count0 = select("label",candidates)
        if count0 > 0:
            return label0, count0

    label0, count0 = select("label",candidates)
    if count0 > 0:
        return label0, count0

    # text() or .
    label0 , count0 = select("title",candidates)
    if count0 >0:
        return label0, count0

    label = candidates[0]
    nhit = len(candidates)
    if verbose:
        print (f'Warning: {nhit} keys found for text():')
        for st in candidates:
            print (f"\t{st}")

    return candidates[0], nhit

def bundle_reference(label,variables):
    candidates = []
    for key in variables:
        if variables[key] == label:
            candidates.append(key)
    return candidates

def generate_bundle(keys,variables=None):
    propfile = ""
    added_keys = {}
    if not variables:
        return propfile
    for key in keys:
        if key in added_keys:
            # print(f"duplicated entries {key}")
            added_keys[key] += 1
            continue
        added_keys[key] = 1
        propfile += f"{key} = {variables[key]}\n"
    return propfile

def write_file(path,buffer,destname):
    dir = os.path.dirname(path)
    if dir and not os.path.exists(dir):
        sys.stderr.write(f"Error: directory {dir} does not exist\n")
        return 0
    with codecs.open(path,'w','utf-8') as outp:
        outp.write(buffer)
    nlines = buffer.count('\n')
    #print (f"{destname} written to \"{path}. {nlines}")
    print (f"{nlines} lines written to \"{path}\".")
    return 1

if __name__ == "__main__":

    text ='''${Obj_DE_ZeroState_Svg}    xpath://*[text()= "No data is selected."] '''
    m = text_pat.search(text)
    if m:
        print (f'{m.group("T")}')
    else:
        print ("not found.")



#!/usr/bin/env python3
import sys
import os

import glob
import re
DIAGNOSE=False

def get_pathlist(*args):
    arglist = []
    options = { "verbose":False, "filetype":None, "prefix":None }

    for arg0 in args:
        if arg0.startswith('verbose='):
            options["verbose"] = False
            if arg0 == "verbose=True":
                options["verbose"] = True
            elif arg0 == 'verbose=False':
                options["verbose"] = False
            else:
                sys.stderr.write("Warning: Invalid option value specified. verbose=[True|False]\n")
            continue
        if arg0.startswith('filetype='):
            arg0 = arg0.split('=')[1]
            if arg0 not in [ "properties", "json", "smd", "tkmsg" , "m", "xml"]:
                sys.stderr.write("Warning: Unsuppoerted filetype specified.\n")
            else:
                options["filetype"] = arg0
            continue
        if arg0.startswith('prefix='):
            arg0 = arg0.split('=')[1]
            options["prefix"] = arg0
            continue
        if not os.path.exists(arg0):
            sys.stderr.write(f"{arg0} does not exist.  Ignored.\n")
            continue
        arglist.append(arg0)
    return arglist, options

#file_pat = re.compile(r'(?P<basename>.*?)((?P<dlm>[_-])(?P<loc>([a-z][a-z]|[a-z][a-z][-_][A-Z][A-Z]|zh[-_](Hans|Hant))))?\.(?P<filetype>\w+)')
file_pat = re.compile(r'(?P<basedir>.*/)?(?P<basename>.*?)((?P<dlm>[_-])(?P<loc>([a-z][a-z]|[a-z][a-z][-_][A-Z][A-Z]|zh[-_](Hans|Hant))))?\.(?P<filetype>\w+)')
dir_pat = re.compile(r'(?P<basedir>.*)(?P<dlm>/)((?P<loc>([a-z][a-z]|[a-z][a-z][-_][a-zA-Z][a-zA-Z]|[a-z][a-z][-_][a-zA-Z][a-zA-Z][-_]pseudo|zh[-_](Hans|Hant))))?/(?P<basename>[a-zA-Z_\.\ -]+?)\.(?P<filetype>\w+)')
#dir_pat = re.compile(r'(?P<basedir>.*)(?P<dlm>/)((?P<loc>([a-z][a-z]|[a-z][a-z][-_][A-Z][A-Z]|zh[-_](Hans|Hant))))?/(?P<basename>.*?)\.(?P<filetype>\w+)')
def locale_lookup(path,user_locale,verbose=False):
    basename = os.path.basename(path)
    filename , fext = os.path.splitext(basename)
    parent = path[:-len(basename)]
    filename_pat = file_pat  # default to lang suffix
    # determine whether file has lang suffix or lang folder
    m = dir_pat.match(path)
    if m:
        filetype = m.group('filetype')
        loc = m.group('loc')
        loc = m.group('dlm')
        parent = m.group('basedir')
        fpat = parent + "/*/" + filename + fext
        filename_pat = dir_pat  # lang folder
        #print (f"debug: language folder found. fpat={fpat} path={path}")
    else:
        fpat = parent+filename+"*"+fext
    # print (f"\tSearchinng basename={basename} fpat={fpat}")
    files = glob.glob(fpat)
    # print (f"files={files}")
    locales = {}
    def normalize_locale(l):
        locale = l.replace("_","-")
        if locale == "zh-Hans":
            locale = "zh"
        elif locale in [ "zh-Hant", "zh-TW" ]:
            locale = "zt"  # use ZT
        elif locale in [ "zh-HK", "zh-TW" , "zh-MO"]:
            locale = l.replace("zh","zt")  # use ZT internally
        # if l != locale:
        #     print (f"{l} --> {locale}")
        return locale

    for file in files:
        m = filename_pat.match(file)
        if m:
            file_locale = m.group("loc")
            # normalize locale delimiter
            if not file_locale:
                file_locale = "en"
            else:
                file_locale = file_locale.replace("_","-")
            #print(f"{file}")
            file_locale = normalize_locale(file_locale)
            locales[file_locale] = { "file":file, "locale":file_locale, "dlm":m.group("dlm")}
        else:
            sys.stderr.write( f"Warning: unknown locale in {file}.\n")
    # find closest locale
    user_locale = normalize_locale(user_locale)
    lang = user_locale.split("-")[0]
    if user_locale in locales: # exact match
        return locales[user_locale]["file"]
    if lang in locales: # language match
        return locales[lang]["file"]
    if "en" in locales:
        return locales["en"]["file"]

    return None

langdir_pat = re.compile('.*/doc/([a-z][a-z])')
def get_basefiles(rootdir,filetype="properties",verbose=False):
    basefiles = {}
    if not os.path.isdir(rootdir):
        fpat0 = os.path.splitext(rootdir)
        fpat = fpat0[0] + "*" + fpat0[1]
    else:
        if filetype:
            fpat = os.path.join(rootdir,f"**/*.{filetype}")
        else:
            fpat = os.path.join(rootdir,'**/*.properties')

    files = glob.glob(fpat,recursive=True)
    nfiles = len(files)
    files.sort()
    print (f"Searching for {fpat} nfiles={nfiles} fileype={filetype}")
    for file in files:
        dirname = os.path.dirname(file)
        basename = os.path.basename(file)
        m = langdir_pat.match(dirname)
        if m:
            if verbose:
                print(f"{dirname} is lang folder.  Ignored.")
            continue

        m = dir_pat.match(file)
        if m:
            ## need to handle pseudo here.
            loc = m.group("loc")
            dirname0 = m.group("basedir")
            basename0 = m.group("basename")
            filetype0 = m.group('filetype')
            if loc == "en":
                basename0 = f"{dirname}/en/{basename0}.{filetype0}"
                basefiles[basename0] = {}
            if loc == "en=xx":
                print (f"en-xx basename= {basename}")
            if basename0 not in basefiles:
                if DIAGNOSE:
                    sys.stderr.write(f"Warning: basefile is missing. \n\t{basename0} ignored.\n")
            else:
                basefiles[basename0][loc] = file
            continue

        m = file_pat.match(basename)
        if m:
            filetype0 = m.group('filetype')
            basename0 = m.group("basename")
            basename0 = f"{dirname}/{basename0}.{filetype0}"
            dlm = m.group('dlm')
            loc = m.group('loc')
            #print (f"basename0={basename0} \n\t{basename} dlm={dlm} loc={loc}")
            if not dlm:
                basefiles[basename0] = {}
                loc = "en"

            if basename0 not in basefiles:
                if DIAGNOSE:
                    sys.stderr.write(f"Warning: basefile is missing. \n\t{basename0} ignored.\n")
            else:
                basefiles[basename0][loc] = file
        else:
            print (f"Unsupported file name {file}.")
    return basefiles



# -*- coding: utf-8 -*-
from codecs import open
from os import path

from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='robotLocalization',
    version='0.8.6',
    packages=find_packages(),
    license='MIT',
    author='Shinichiro Kayano',
    description='Robot Framework Internationalization',
    long_description=long_description,
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Environment :: Console',
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Framework :: Robot Framework :: Tool',
        'Topic :: Software Development :: Localization'
    ],
    install_requires=['jproperties>=2.1.1', 'requests>=2.22.0'],
    extras_require={
        'dev': ["pytest", "docutils", "pygments"],
        'optional': ["robotframework", "robotframework-seleniumlibrary" ,
            "robotframework-autorecorder", "robotframework-requests" , "opencv-python"],
    },
    keywords="localization cicd test automation",
    entry_points={
        'console_scripts': [
            'robotLocalization=robotLocalization:main',
        ],
    },

)
